<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class RelImpostoPorProdutoPorEmissaoService {

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function find($conditions) {
        $params = array(
         	":P_CDFILIAL" => $conditions['CDFILIAL'],
         	":P_TCDFILIAL" => $conditions['TCDFILIAL'],
            ":P_CDCAIXA" => $conditions['CDCAIXA'],
            ":P_TCDCAIXA" => $conditions['TCDCAIXA'],
         	":P_CDIMPOSTO" => $conditions['CDIMPOSTO'],
         	":P_TCDIMPOSTO" => $conditions['TCDIMPOSTO'],
         	":P_DTVENDAINI" => $conditions['DTVENDAINI'],
         	":P_DTVENDAFIN" => $conditions['DTVENDAFIN'],
         	":CDARVPRODF" => $conditions['CDARVPRODF'],
         	":CDARVPRODI" => $conditions['CDARVPRODI'],
         	":CDPRODLIST" => $conditions['CDPRODLIST'],
         	":PRODNIV" => $conditions['PRODNIV'],
         	":TCDPRODUTO" => $conditions['TCDPRODUTO']
 		);
 		return $this->entityManager->getConnection()->fetchAll(self::GET_IMPOSTO_PRODUTO_EMISSAO, $params);
    }
    
    public function buildParams($conditions){
        $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
        $conditions['TCDFILIAL'] = 'E';
        if( array_key_exists('CDFILICAIXA', $conditions) ) {
            $conditions['CDCAIXA'] = implode('_', $conditions['CDFILICAIXA']);
            $conditions['TCDCAIXA'] = 'F';
            unset($conditions['CDFILICAIXA']);
        }  
        else {
            $conditions['CDCAIXA'] = null;
            $conditions['TCDCAIXA'] = 'T';    
        }
        if(array_key_exists('DTVENDA', $conditions)) {
            $conditions['DTVENDAINI'] = $conditions['DTVENDA'][0];
            $conditions['DTVENDAFIN'] = $conditions['DTVENDA'][1];
            unset($conditions['DTVENDA']);
        }
        if (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'I'){
           if (empty($conditions['CDARVPRODF'])){
                $conditions['CDARVPRODF'] = $conditions['CDARVPRODI'];
           }
           $conditions['TCDPRODUTO'] = null;
           $conditions['CDPRODLIST'] = null;
           $conditions['PRODNIV'] = null;
        }elseif (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'D') {
           $conditions['PRODNIV'] = str_replace('%', '', str_replace('.', '', $conditions['PRODNIV'])).'%';
           $conditions['TCDPRODUTO'] = null;
           $conditions['CDPRODLIST'] = null;
           $conditions['CDARVPRODF'] = null;
           $conditions['CDARVPRODI'] = null;
        }
        elseif (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'L'){
           $conditions['CDPRODLIST'] = implode('_', $conditions['CDPRODUTO']);
           $conditions['TCDPRODUTO'] = null;
           $conditions['CDARVPRODF'] = null;
           $conditions['CDARVPRODI'] = null;
           $conditions['PRODNIV'] = null;
        }
        else{
           $conditions['TCDPRODUTO'] = 'T';
           $conditions['CDPRODLIST'] = null;
           $conditions['CDARVPRODF'] = null;
           $conditions['CDARVPRODI'] = null;
           $conditions['PRODNIV'] = null;
        }
        if(array_key_exists('CDIMPOSTOPROD', $conditions) ) {
            $conditions['CDIMPOSTO'] = implode('_',$conditions['CDIMPOSTOPROD']);
            $conditions['TCDIMPOSTO'] = 'F';
            unset($conditions['CDIMPOSTOPROD']);
        }  
        else {
            $conditions['CDIMPOSTO'] = null;
            $conditions['TCDIMPOSTO'] = 'T';    
        }
        return $conditions;
    }

    const GET_IMPOSTO_PRODUTO_EMISSAO = "
        SELECT FILTROVENDA.CDFILIAL, FILTROVENDA.NMFILIAL, FILTROVENDA.CDFILIAL || ' - ' || FILTROVENDA.NMFILIAL AS LABELFILIAL, FILTROVENDA.CDCAIXA || ' - ' || FILTROVENDA.NMCAIXA AS CDFILICAIXA, IMP.CDIMPOSTO || ' - ' || IMP.NMIMPOSTO AS LABELIMPOSTO, PR.CDARVPROD, PR.NMPRODUTO, PR.SGUNIDADE,
               II.VRPEALPRODIT,  II.VRPERPIS, II.VRPERCOFINS, SUM(IT.QTPRODVEND) AS QTPRODVEND,
               SUM(TRUNC((IT.QTPRODVEND * (IT.VRUNITVEND + NVL(IT.VRUNITVENDCL,0))),2) - NVL(IT.VRDESITVEND ,0) + NVL(IT.VRACRITVEND,0)) AS VRPRODVEN,
               SUM(NVL(II.VRBASECALCREDUZ,0)) AS VRBASECALCREDUZ,
               SUM(VRBASECALCICMS) AS VRBCICMS,
               SUM(VRIMPOPRODIT) AS VRICMS,
               SUM(VRBASECALCICMS) AS VRBCPIS,
               SUM(VRIMPPIS) AS VRPIS,
               SUM(VRBASECALCICMS) AS VRBCCOFINS,
               SUM(VRIMPCOFINS) AS VRCOFINS,
               DECODE(FILTROVENDA.IDTPEMISVEND, 'E', 'ECF', '', 'ECF', 'N', 'NFCE', 'S', 'SAT') TIPO
          FROM (SELECT DISTINCT V.CDCAIXA, FLC.NMCAIXA, V.CDFILIAL, V.NRSEQVENDA, IDTPEMISVEND, FLC.NMFILIAL
                  FROM VENDA V, V_FILIALLOJACAIXA FLC
                 WHERE V.CDFILIAL = FLC.CDFILIAL
                   AND V.CDCAIXA = FLC.CDCAIXA
                   AND V.CDLOJA = FLC.CDLOJA
                   AND V.IDSITUVENDA  = 'O'
                   AND ((INSTR(:P_CDFILIAL , FLC.CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                   AND ((INSTR(:P_CDCAIXA  , FLC.CDFILIAL||FLC.CDCAIXA ) <> 0) OR (:P_TCDCAIXA = 'T'))
                   AND ((V.IDTPEMISVEND IN ('S','N') AND V.IDTPAMBNFCE = '1' AND TRUNC(V.DTEMISSAONFCE) BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN) OR
                       (((V.IDTPEMISVEND = 'E') OR (V.IDTPEMISVEND IS NULL)) AND V.DTENTRVENDA BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN))) FILTROVENDA,
               (SELECT PR.CDARVPROD, PR.NMPRODUTO, PR.SGUNIDADE, PR.CDPRODUTO
                  FROM PRODUTO PR
                 WHERE (((((INSTR(:CDPRODLIST, PR.CDPRODUTO) > 0 ) OR (PR.CDARVPROD BETWEEN :CDARVPRODI AND :CDARVPRODF ))) OR (PR.CDARVPROD LIKE :PRODNIV)) OR (:TCDPRODUTO = 'T'))) PR,
               (SELECT CDIMPOSTO, NMIMPOSTO
                  FROM IMPOSTO
                 WHERE ((INSTR(:P_CDIMPOSTO, CDIMPOSTO ) <> 0) OR (:P_TCDIMPOSTO = 'T'))) IMP,
                ITEMVENDA IT, ITVENDAIMPOS II
          WHERE FILTROVENDA.CDFILIAL = IT.CDFILIAL
            AND FILTROVENDA.CDCAIXA = IT.CDCAIXA
            AND FILTROVENDA.NRSEQVENDA = IT.NRSEQVENDA
            AND IT.CDPRODUTO = PR.CDPRODUTO
            AND IT.CDFILIAL = II.CDFILIAL
            AND IT.CDCAIXA = II.CDCAIXA
            AND IT.NRSEQVENDA = II.NRSEQVENDA
            AND IT.NRSEQUITVEND = II.NRSEQUITVEND
            AND II.CDIMPOSTO = IMP.CDIMPOSTO
          GROUP BY FILTROVENDA.CDFILIAL, FILTROVENDA.NMFILIAL, FILTROVENDA.CDCAIXA, FILTROVENDA.NMCAIXA , IMP.CDIMPOSTO, IMP.NMIMPOSTO, PR.CDARVPROD, PR.NMPRODUTO, PR.SGUNIDADE, II.VRPEALPRODIT, II.VRPERPIS, II.VRPERCOFINS, DECODE(FILTROVENDA.IDTPEMISVEND, 'E', 'ECF', '', 'ECF', 'N', 'NFCE', 'S', 'SAT')
          ORDER BY FILTROVENDA.CDFILIAL, PR.NMPRODUTO
    ";
}